import { Tree, SyntaxNode, TreeFragment, NodeProp, Parser } from "@lezer/common";
import type { LRParser, ParserConfig } from "@lezer/lr";
import { EditorState, StateField, Transaction, Extension, Facet, ChangeDesc } from "@codemirror/state";
import { EditorView } from "@codemirror/view";
export declare const languageDataProp: NodeProp<Facet<{
    [name: string]: any;
}, readonly {
    [name: string]: any;
}[]>>;
export declare function defineLanguageFacet(baseData?: {
    [name: string]: any;
}): Facet<{
    [name: string]: any;
}, readonly {
    [name: string]: any;
}[]>;
export interface Sublanguage {
    type?: "replace" | "extend";
    test: (node: SyntaxNode, state: EditorState) => boolean;
    facet: Facet<{
        [name: string]: any;
    }>;
}
export declare const sublanguageProp: NodeProp<Sublanguage[]>;
export declare class Language {
    readonly data: Facet<{
        [name: string]: any;
    }>;
    readonly name: string;
    readonly extension: Extension;
    parser: Parser;
    constructor(data: Facet<{
        [name: string]: any;
    }>, parser: Parser, extraExtensions?: Extension[], name?: string);
    isActiveAt(state: EditorState, pos: number, side?: -1 | 0 | 1): boolean;
    findRegions(state: EditorState): {
        from: number;
        to: number;
    }[];
    get allowsNesting(): boolean;
    static state: StateField<LanguageState>;
    static setState: import("@codemirror/state").StateEffectType<LanguageState>;
}
export declare class LRLanguage extends Language {
    readonly parser: LRParser;
    private constructor();
    static define(spec: {
        name?: string;
        parser: LRParser;
        languageData?: {
            [name: string]: any;
        };
    }): LRLanguage;
    configure(options: ParserConfig, name?: string): LRLanguage;
    get allowsNesting(): boolean;
}
export declare function syntaxTree(state: EditorState): Tree;
export declare function ensureSyntaxTree(state: EditorState, upto: number, timeout?: number): Tree | null;
export declare function syntaxTreeAvailable(state: EditorState, upto?: number): boolean;
export declare function forceParsing(view: EditorView, upto?: number, timeout?: number): boolean;
export declare function syntaxParserRunning(view: EditorView): boolean;
export declare class ParseContext {
    private parser;
    readonly state: EditorState;
    fragments: readonly TreeFragment[];
    tree: Tree;
    treeLen: number;
    viewport: {
        from: number;
        to: number;
    };
    skipped: {
        from: number;
        to: number;
    }[];
    scheduleOn: Promise<unknown> | null;
    private parse;
    tempSkipped: {
        from: number;
        to: number;
    }[];
    private constructor();
    static create(parser: Parser, state: EditorState, viewport: {
        from: number;
        to: number;
    }): ParseContext;
    private startParse;
    work(until: number | (() => boolean), upto?: number): boolean;
    takeTree(): void;
    private withContext;
    private withoutTempSkipped;
    changes(changes: ChangeDesc, newState: EditorState): ParseContext;
    updateViewport(viewport: {
        from: number;
        to: number;
    }): boolean;
    reset(): void;
    skipUntilInView(from: number, to: number): void;
    static getSkippingParser(until?: Promise<unknown>): Parser;
    isDone(upto: number): boolean;
    static get(): ParseContext;
}
declare class LanguageState {
    readonly context: ParseContext;
    readonly tree: Tree;
    constructor(context: ParseContext);
    apply(tr: Transaction): LanguageState;
    static init(state: EditorState): LanguageState;
}
export declare const language: Facet<Language, Language>;
export declare class LanguageSupport {
    readonly language: Language;
    readonly support: Extension;
    extension: Extension;
    constructor(language: Language, support?: Extension);
}
export declare class LanguageDescription {
    readonly name: string;
    readonly alias: readonly string[];
    readonly extensions: readonly string[];
    readonly filename: RegExp | undefined;
    private loadFunc;
    support: LanguageSupport | undefined;
    private loading;
    private constructor();
    load(): Promise<LanguageSupport>;
    static of(spec: {
        name: string;
        alias?: readonly string[];
        extensions?: readonly string[];
        filename?: RegExp;
        load?: () => Promise<LanguageSupport>;
        support?: LanguageSupport;
    }): LanguageDescription;
    static matchFilename(descs: readonly LanguageDescription[], filename: string): LanguageDescription;
    static matchLanguageName(descs: readonly LanguageDescription[], name: string, fuzzy?: boolean): LanguageDescription;
}
export {};
